@@ -1,6 +1,6 @@
 /********************************************
 bi_funct.c
-copyright 2008-2021,2023, Thomas E. Dickey
+copyright 2008-2023,2024, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: bi_funct.c,v 1.118 2023/07/24 08:23:33 tom Exp $
+ * $MawkId: bi_funct.c,v 1.127 2024/01/23 21:22:25 tom Exp $
  */
 
 #include <mawk.h>
@@ -167,7 +167,7 @@ str_str(char *target, size_t target_len, const char *key, size_t key_len)
 	while (target_len >= key_len && (target = memchr(target, k, target_len))) {
 	    target_len = target_len - (size_t) (target - prior) - 1;
 	    prior = ++target;
-	    if (target[0] == k1) {
+	    if (target_len > 0 && target[0] == k1) {
 		result = target - 1;
 		break;
 	    }
@@ -179,7 +179,7 @@ str_str(char *target, size_t target_len, const char *key, size_t key_len)
 	while (target_len > key_len && (target = memchr(target, k, target_len))) {
 	    target_len = target_len - (size_t) (target - prior) - 1;
 	    prior = ++target;
-	    if (memcmp(target, key + 1, key_len) == 0) {
+	    if (target_len >= key_len && memcmp(target, key + 1, key_len) == 0) {
 		result = target - 1;
 		break;
 	    }
@@ -343,53 +343,34 @@ bi_match(CELL *sp)
     return_CELL("bi_match", sp);
 }
 
-CELL *
-bi_toupper(CELL *sp)
-{
-    STRING *old;
-    register char *p, *q;
-
-    TRACE_FUNC("bi_toupper", sp);
-
-    if (sp->type != C_STRING)
-	cast1_to_s(sp);
-    old = string(sp);
-    sp->ptr = (PTR) new_STRING0(old->len);
-
-    q = string(sp)->str;
-    p = old->str;
-    while (*p) {
-	*q = *p++;
-	*q = (char) toupper((UChar) * q);
-	q++;
-    }
-    free_STRING(old);
-    return_CELL("bi_toupper", sp);
-}
-
-CELL *
-bi_tolower(CELL *sp)
-{
-    STRING *old;
-    register char *p, *q;
-
-    TRACE_FUNC("bi_tolower", sp);
-
-    if (sp->type != C_STRING)
-	cast1_to_s(sp);
-    old = string(sp);
-    sp->ptr = (PTR) new_STRING0(old->len);
-
-    q = string(sp)->str;
-    p = old->str;
-    while (*p) {
-	*q = *p++;
-	*q = (char) tolower((UChar) * q);
-	q++;
-    }
-    free_STRING(old);
-    return_CELL("bi_tolower", sp);
+#define BI_TOCASE(case) \
+CELL * \
+bi_to##case(CELL *sp) \
+{ \
+    STRING *old; \
+    size_t len; \
+    register char *p, *q; \
+\
+    TRACE_FUNC("bi_to" #case, sp); \
+\
+    if (sp->type != C_STRING) \
+        cast1_to_s(sp); \
+    old = string(sp); \
+    len = old->len; \
+    sp->ptr = (PTR) new_STRING0(len); \
+\
+    q = string(sp)->str; \
+    p = old->str; \
+    while (len--) \
+	*q++ = (char) to##case((UChar) *p++); \
+    free_STRING(old); \
+    return_CELL("bi_to" #case, sp); \
 }
+/* *INDENT-OFF* */
+BI_TOCASE(upper)
+BI_TOCASE(lower)
+#undef BI_TOCASE
+/* *INDENT-ON* */
 
 /*
  * Like gawk...
@@ -499,7 +480,7 @@ bi_strftime(CELL *sp)
     if (n_args > 1) {
 	if (sp[1].type != C_DOUBLE)
 	    cast1_to_d(sp + 1);
-	rawtime = d_to_i(sp[1].dval);
+	rawtime = d_to_l(sp[1].dval);
     } else {
 	time(&rawtime);
     }
@@ -786,11 +767,11 @@ CELL *
 bi_srand(CELL *sp)
 {
 #ifdef USE_SYSTEM_SRAND
-    static long seed = 1;
     static CELL cseed =
     {
 	C_DOUBLE, 0, 0, 1.0
     };
+    double seed32;
 #endif
 
     CELL c;
@@ -814,8 +795,8 @@ bi_srand(CELL *sp)
     }
 
 #ifdef USE_SYSTEM_SRAND
-    seed = d_to_i(cseed.dval);
-    mawk_srand((unsigned) seed);
+    seed32 = fmod(cseed.dval, (double) Max_UInt);
+    mawk_srand((unsigned) seed32);
 #else
     /* The old seed is now in *sp ; move the value in cseed to
        seed in range [1,M) */
@@ -825,7 +806,7 @@ bi_srand(CELL *sp)
 	cast1_to_d(&c);
 
     seed = ((c.type == C_DOUBLE)
-	    ? (long) (d_to_i(c.dval) & M) % M + 1
+	    ? (long) (d_to_l(c.dval) & M) % M + 1
 	    : (long) hash(string(&c)->str) % M + 1);
     if (seed == M)
 	seed = M - 1;
@@ -918,7 +899,6 @@ bi_fflush(CELL *sp)
 CELL *
 bi_system(CELL *sp GCC_UNUSED)
 {
-#ifdef HAVE_REAL_PIPES
     int ret_val;
 
     TRACE_FUNC("bi_system", sp);
@@ -926,24 +906,22 @@ bi_system(CELL *sp GCC_UNUSED)
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
 
+#ifdef HAVE_REAL_PIPES
     flush_all_output();
     ret_val = wait_status(system(string(sp)->str));
+#elif defined(__MINGW32__)
+    flush_all_output();
+    ret_val = system(string(sp)->str);
+#elif defined(MSDOS)
+    ret_val = DOSexec(string(sp)->str);
+#else
+    ret_val = -1;
+#endif
 
     cell_destroy(sp);
     sp->type = C_DOUBLE;
     sp->dval = (double) ret_val;
-#elif defined(MSDOS)
-    int retval;
 
-    if (sp->type < C_STRING)
-	cast1_to_s(sp);
-    retval = DOSexec(string(sp)->str);
-    free_STRING(string(sp));
-    sp->type = C_DOUBLE;
-    sp->dval = (double) retval;
-#else
-    sp = 0;
-#endif
     return_CELL("bi_system", sp);
 }
 
@@ -988,7 +966,7 @@ bi_getline(CELL *sp)
 	sp--;
 	if (sp->type < C_STRING)
 	    cast1_to_s(sp);
-	fin_p = (FIN *) file_find(sp->ptr, F_IN);
+	fin_p = (FIN *) file_find(string(sp), F_IN);
 	free_STRING(string(sp));
 	sp--;
 
@@ -1005,7 +983,7 @@ bi_getline(CELL *sp)
 	sp -= 2;
 	if (sp->type < C_STRING)
 	    cast1_to_s(sp);
-	fin_p = (FIN *) file_find(sp->ptr, PIPE_IN);
+	fin_p = (FIN *) file_find(string(sp), PIPE_IN);
 	free_STRING(string(sp));
 
 	if (!fin_p)
