@@ -1,6 +1,6 @@
 /********************************************
 code.c
-copyright 2009-2016,2019, Thomas E. Dickey
+copyright 2009-2019,2023, Thomas E. Dickey
 copyright 1991-1994,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,18 +11,18 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: code.c,v 1.39 2019/02/02 01:09:45 tom Exp $
+ * $MawkId: code.c,v 1.44 2023/08/15 23:24:13 tom Exp $
  */
 
-#include "mawk.h"
-#include "code.h"
-#include "init.h"
-#include "jmp.h"
-#include "field.h"
+#include <mawk.h>
+#include <code.h>
+#include <init.h>
+#include <jmp.h>
+#include <field.h>
 
 #ifdef NO_LEAKS
-#include "repl.h"
-#include "scan.h"
+#include <repl.h>
+#include <scan.h>
 #endif
 
 static CODEBLOCK *new_code(void);
@@ -40,9 +40,9 @@ INST *execution_start = 0;
 void
 code_grow(void)
 {
-    unsigned oldsize = (unsigned) (code_limit - code_base);
-    unsigned newsize = PAGESZ + oldsize;
-    unsigned delta = (unsigned) (code_ptr - code_base);
+    size_t oldsize = (size_t) (code_limit - code_base);
+    size_t newsize = PAGESZ + oldsize;
+    size_t delta = (size_t) (code_ptr - code_base);
 
     if (code_ptr > code_limit)
 	bozo("CODEWARN is too small");
@@ -69,9 +69,9 @@ code_shrink(CODEBLOCK * p, size_t *sizep)
     retval = (INST *) zrealloc(p->base, oldsize, newsize);
     TRACE(("code_shrink old %p %lu, new %p %lu\n",
 	   (void *) p->base,
-	   oldsize,
+	   (unsigned long) oldsize,
 	   (void *) retval,
-	   newsize));
+	   (unsigned long) newsize));
     ZFREE(p);
     return retval;
 }
@@ -210,12 +210,20 @@ be_setup(int scope)
     *main_code_p = active_code;
 
     if (scope == SCOPE_BEGIN) {
-	if (!begin_code_p)
+	if (!begin_code_p) {
+	    TRACE(("be_setup: BEGIN\n"));
 	    begin_code_p = new_code();
+	} else {
+	    TRACE(("be_setup: BEGIN (again)\n"));
+	}
 	active_code = *begin_code_p;
     } else {
-	if (!end_code_p)
+	if (!end_code_p) {
+	    TRACE(("be_setup: END\n"));
 	    end_code_p = new_code();
+	} else {
+	    TRACE(("be_setup: END (again)\n"));
+	}
 	active_code = *end_code_p;
     }
 }
@@ -259,10 +267,16 @@ free_codes(const char *tag, INST * base, size_t size)
 
     (void) tag;
 
-    TRACE(("free_codes(%s) base %p, size %lu\n", tag, (void *) base, size));
+    TRACE(("free_codes(%s) base %p, size %lu\n",
+	   tag,
+	   (void *) base,
+	   (unsigned long) size));
+
     if (base != 0 && size != 0) {
 	for (cdp = base; cdp < last; ++cdp) {
+#ifndef NO_LEAKS
 	    TRACE_INST(cdp, base);
+#endif
 
 	    switch ((MAWK_OPCODES) (cdp->op)) {
 	    case AE_PUSHA:
@@ -286,6 +300,7 @@ free_codes(const char *tag, INST * base, size_t size)
 	    case A_PUSHA:
 	    case L_PUSHA:
 	    case L_PUSHI:
+	    case _LENGTH:
 	    case _BUILTIN:
 	    case _PRINT:
 	    case _PUSHA:
@@ -309,6 +324,7 @@ free_codes(const char *tag, INST * base, size_t size)
 	    case _RANGE:
 		cdp += 4;	/* PAT1 */
 		break;
+	    case _CALLX:
 	    case _CALL:
 		cdp += 1 + cdp[2].op;
 		break;
