@@ -1,6 +1,6 @@
 /********************************************
 code.h
-copyright 2009-2012,2019, Thomas E. Dickey
+copyright 2009-2019,2023, Thomas E. Dickey
 copyright 1991-1994,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: code.h,v 1.11 2019/01/30 00:49:25 tom Exp $
+ * $MawkId: code.h,v 1.14 2023/08/16 23:34:11 tom Exp $
  */
 
 /*  code.h  */
@@ -19,7 +19,7 @@ the GNU General Public License, version 2, 1991.
 #ifndef  MAWK_CODE_H
 #define  MAWK_CODE_H
 
-#include "memory.h"
+#include <memory.h>
 
 #define  PAGESZ	512
 	/* number of code instructions allocated at one time */
@@ -103,8 +103,9 @@ typedef enum {
     ,_POW
     ,_NOT
     ,_TEST
-    ,A_LENGTH
     ,A_TEST
+    ,_LENGTH
+    ,A_LENGTH
     ,A_DEL
     ,ALOOP
     ,A_CAT
@@ -155,6 +156,7 @@ typedef enum {
     ,_NEXTFILE
     ,_RANGE
     ,_CALL
+    ,_CALLX
     ,_RET
     ,_RET0
     ,SET_ALOOP
