@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: execute.c,v 1.48 2023/07/25 22:25:27 tom Exp $
+ * $MawkId: execute.c,v 1.54 2023/11/27 00:48:08 tom Exp $
  */
 
 #include <mawk.h>
@@ -108,7 +108,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 				   user defined functions */
 {
     static INST *restart_label;	/* control flow label */
-    static CELL tc;		/*useful temp */
+    static CELL tc;		/* useful temp */
     static CELL missing;	/* no value (use zero) */
 
     /* some useful temporaries */
@@ -219,14 +219,18 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	case L_PUSHI:
 	    /* put the contents of a local var on stack,
 	       cdp->op holds the offset from the frame pointer */
-	    inc_sp();
-	    cellcpy(sp, fp + (cdp++)->op);
+	    if (fp != NULL) {
+		inc_sp();
+		cellcpy(sp, fp + (cdp++)->op);
+	    }
 	    break;
 
 	case L_PUSHA:
 	    /* put a local address on eval stack */
-	    inc_sp();
-	    sp->ptr = (PTR) (fp + (cdp++)->op);
+	    if (fp != NULL) {
+		inc_sp();
+		sp->ptr = (PTR) (fp + (cdp++)->op);
+	    }
 	    break;
 
 	case F_PUSHI:
@@ -357,7 +361,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    {
 		SYMTAB *stp = (SYMTAB *) cdp->ptr;
 		cdp--;
-		TRACE(("patching %s\n", type_to_str(stp->type)));
+		TRACE(("patch/alen %s\n", type_to_str(stp->type)));
 		switch (stp->type) {
 		case ST_VAR:
 		    cdp[0].op = _PUSHI;
@@ -391,7 +395,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 		ZFREE(dl);
 		cdp--;
-		TRACE(("patching %s\n", type_to_str(type)));
+		TRACE(("patch/len %s\n", type_to_str(type)));
 		switch (type) {
 		case ST_LOCAL_VAR:
 		    cdp[0].op = L_PUSHI;
@@ -1195,6 +1199,8 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		} else {
 		    set_field0(p, len);
 		    cdp = restart_label;
+		    if (cdp == NULL)
+			bozo("empty restart-label");
 		    rt_nr++;
 		    rt_fnr++;
 		}
@@ -1218,6 +1224,8 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		} else {
 		    set_field0(p, len);
 		    cdp = restart_label;
+		    if (cdp == NULL)
+			bozo("empty restart-label");
 
 		    if (TEST2(NR) != TWO_DOUBLES)
 			cast2_to_d(NR);
@@ -1294,6 +1302,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 	    return;
 
+	case _CALLX:
 	case _CALL:
 
 	    /*  cdp[0] holds ptr to "function block"
@@ -1305,20 +1314,23 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		int a_args = (cdp++)->op;	/* actual number of args */
 		CELL *nfp = sp - a_args + 1;	/* new fp for callee */
 		CELL *local_p = sp + 1;		/* first local argument on stack */
-		char *type_p = 0;	/* pts to type of an argument */
+		SYM_TYPE *type_p = 0;	/* pts to type of an argument */
 
-		if (fbp->nargs)
+		if (fbp->nargs) {
 		    type_p = fbp->typev + a_args - 1;
 
-		/* create space for locals */
-		t = fbp->nargs - a_args;	/* t is number of locals */
-		while (t > 0) {
-		    t--;
-		    inc_sp();
-		    type_p++;
-		    sp->type = C_NOINIT;
-		    if ((type_p) != 0 && (*type_p == ST_LOCAL_ARRAY))
-			sp->ptr = (PTR) new_ARRAY();
+		    /* create space for locals */
+		    t = fbp->nargs - a_args;	/* t is number of locals */
+		    while (t > 0) {
+			t--;
+			inc_sp();
+			type_p++;
+			if (*type_p == ST_LOCAL_ARRAY) {
+			    sp->ptr = (PTR) new_ARRAY();
+			} else {
+			    sp->type = C_NOINIT;
+			}
+		    }
 		}
 
 		execute(fbp->code, sp, nfp);
