@@ -1,6 +1,6 @@
 /********************************************
 field.c
-copyright 2008-2016,2020 Thomas E. Dickey
+copyright 2008-2021,2023 Thomas E. Dickey
 copyright 1991-1995,2014 Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: field.c,v 1.36 2020/01/20 11:47:26 tom Exp $
+ * $MawkId: field.c,v 1.41 2023/07/23 11:33:56 tom Exp $
  */
 
 /* field.c */
@@ -106,7 +106,7 @@ set_rs_shadow(void)
     CELL c;
     STRING *sval;
     char *s;
-    SLen len;
+    size_t len;
 
     if (posix_space_flag && mawk_state == EXECUTION)
 	scan_code['\n'] = SC_UNEXPECTED;
@@ -207,7 +207,7 @@ field_init(void)
 }
 
 void
-set_field0(char *s, size_t len)
+set_field0(const char *s, size_t len)
 {
     cell_destroy(&field[0]);
     nf = -1;
@@ -517,8 +517,8 @@ build_field0(void)
 	STRING *ofs, *tail;
 	size_t len;
 	register CELL *cp;
-	register char *p, *q;
 	int cnt;
+	register char *p;
 	CELL **fbp, *cp_limit;
 
 	cast1_to_s(cellcpy(&c, OFS));
@@ -539,12 +539,12 @@ build_field0(void)
 		    cp->ptr = (PTR) & null_str;
 		    null_str.ref_cnt++;
 		} else {	/* its a double */
-		    Int ival;
+		    Long ival;
 		    char xbuff[260];
 
-		    ival = d_to_I(cp->dval);
+		    ival = d_to_L(cp->dval);
 		    if (ival == cp->dval)
-			sprintf(xbuff, INT_FMT, ival);
+			sprintf(xbuff, LONG_FMT, ival);
 		    else
 			sprintf(xbuff, string(CONVFMT)->str, cp->dval);
 
@@ -571,7 +571,10 @@ build_field0(void)
 	fbp = fbankv;
 	cp = field + 1;
 	cp_limit = field + FBANK_SZ;
+
 	while (cnt-- > 0) {
+	    register char *q;
+
 	    memcpy(p, string(cp)->str, string(cp)->len);
 	    p += string(cp)->len;
 	    /* if not really string, free temp use of ptr */
@@ -612,11 +615,12 @@ slow_cell_assign(CELL *target, CELL *source)
 	size_t i;
 	for (i = 1; i < fbankv_num_chunks * FBANKV_CHUNK_SIZE; i++) {
 	    CELL *bank_start = fbankv[i];
-	    CELL *bank_end = bank_start + FBANK_SZ;
+	    CELL *bank_end;
 
 	    if (bank_start == 0)
 		break;
 
+	    bank_end = bank_start + FBANK_SZ;
 	    if (bank_start <= target && target < bank_end) {
 		/* it is a field */
 		field_assign(target, source);
@@ -689,7 +693,7 @@ binmode(void)
    from environment or -W binmode=   */
 
 void
-set_binmode(int x)
+set_binmode(long x)
 {
     CELL c;
     int change = ((x & 4) == 0);
