@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: fin.h,v 1.14 2023/07/22 22:27:30 tom Exp $
+ * $MawkId: fin.h,v 1.15 2023/08/08 21:40:03 tom Exp $
  */
 
 /* fin.h */
@@ -29,7 +29,7 @@ typedef struct {
     char *buff;			/* base of data read from file */
     char *buffp;		/* current position to read-next */
     char *limit;		/* points past the data in *buff */
-    unsigned nbuffs;		/* sizeof *buff in BUFFSZs */
+    size_t buff_size;		/* allocated size of buff[] */
     int flags;
 } FIN;
 
@@ -38,14 +38,14 @@ typedef struct {
 #define  START_FLAG   4		/* used when RS == "" */
 #define  FIN_FLAG     8		/* set if fin->buff is no longer beginning */
 
-FIN *FINdopen(int, int);
-FIN *FINopen(char *, int);
-void FINclose(FIN *);
-void FINsemi_close(FIN *);
-char *FINgets(FIN *, size_t *);
-size_t fillbuff(int, char *, size_t);
+extern FIN *FINdopen(int, int);
+extern FIN *FINopen(char *, int);
+extern void FINclose(FIN *);
+extern void FINsemi_close(FIN *);
+extern char *FINgets(FIN *, size_t *);
+extern size_t fillbuff(int, char *, size_t);
+extern void open_main(void);
 
 extern FIN *main_fin;		/* for the main input stream */
-void open_main(void);
 
 #endif /* FIN_H */
