@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: mawk.h,v 1.63 2023/07/20 00:11:29 tom Exp $
+ * $MawkId: mawk.h,v 1.67 2023/11/27 01:03:52 tom Exp $
  */
 
 /*  mawk.h  */
@@ -19,7 +19,7 @@ the GNU General Public License, version 2, 1991.
 #ifndef  MAWK_H
 #define  MAWK_H
 
-#include "nstd.h"
+#include <nstd.h>
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -30,7 +30,7 @@ the GNU General Public License, version 2, 1991.
 
 #include <assert.h>
 
-#include "types.h"
+#include <types.h>
 
 #ifdef HAVE_STDNORETURN_H
 #include <stdnoreturn.h>
@@ -127,7 +127,11 @@ extern unsigned rt_nr, rt_fnr;	/* ditto */
 	do { \
 	    if ( (cp)->type >= C_STRING && \
 	         (cp)->type <= C_MBSTRN ) { \
+	        unsigned final = string(cp)->ref_cnt; \
 		free_STRING(string(cp));  \
+		if (final <= 1) { \
+		    (cp)->ptr = NULL; \
+		} \
 	    } \
 	} while (0)
 #endif
@@ -189,6 +193,10 @@ extern double strtod_with_ovf_bug(const char *, char **);
 #define strtod  strtod_with_ovf_bug
 #endif
 
+#ifndef OPT_CALLX
+#define OPT_CALLX 0
+#endif
+
 #if OPT_TRACE > 0
 extern void Trace(const char *, ...) GCC_PRINTFLIKE(1,2);
 extern void TraceVA(const char *, va_list);
@@ -233,10 +241,8 @@ extern void TraceInst(INST *, INST *);
 #define TRACE_INST(cp,base)	/* nothing */
 #endif
 
-#if OPT_TRACE > 0
 extern const char *da_type_name(CELL *);
 extern const char *da_op_name(INST *);
-#endif
 
 #ifdef NO_LEAKS
 
