@@ -1,6 +1,6 @@
 /*
 regexp_system.c
-copyright 2009-2010,2014, Thomas E. Dickey
+copyright 2009-2014,2020, Thomas E. Dickey
 copyright 2005, Aleksey Cheusov
 
 This is a source file for mawk, an implementation of
@@ -10,8 +10,12 @@ Mawk is distributed without warranty under the terms of
 the GNU General Public License, version 2, 1991.
  */
 
-/* $MawkId: regexp.c,v 1.8 2014/08/08 08:15:15 tom Exp $ */
+/* $MawkId: regexp.c,v 1.10 2020/10/16 22:27:03 tom Exp $ */
+
+#include <config.h>
+
 #ifdef LOCAL_REGEXP
+#define REGEXP_INTERNALS
 #		include "mawk.h"
 #define RE_FILL() { goto refill; }
 #define RE_CASE() { goto reswitch; }
