@@ -1,6 +1,6 @@
 /********************************************
 regexp.h
-copyright 2009-2010,2014, Thomas E. Dickey
+copyright 2009-2014,2020, Thomas E. Dickey
 copyright 2005, Aleksey Cheusov
 copyright 1991,1993, Michael D. Brennan
 
@@ -12,14 +12,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: regexp.h,v 1.13 2014/09/11 23:43:10 tom Exp $
- * @Log: regexp.h,v @
- * Revision 1.1.1.1  1993/07/03  18:58:19  mike
- * move source to cvs
- *
- * Revision 5.1  1991/12/05  07:59:30  brennan
- * 1.1 pre-release
- *
+ * $MawkId: regexp.h,v 1.15 2020/10/16 22:43:29 tom Exp $
  */
 #ifndef  MAWK_REPL_H
 #define  MAWK_REPL_H
@@ -27,12 +20,22 @@ the GNU General Public License, version 2, 1991.
 #include <stdio.h>
 #include "nstd.h"
 
+const char *REerror(void);
+
+#ifdef LOCAL_REGEXP
+#include "rexp.h"
+STATE *REcompile(char *, size_t);
+void REdestroy(STATE *);
+int REtest(char *, size_t, STATE *);
+char *REmatch(char *, size_t, STATE *, size_t *, int);
+void REmprint(STATE *, FILE *);
+const char *REs_type(STATE *);
+#else
 PTR REcompile(char *, size_t);
 void REdestroy(PTR);
 int REtest(char *, size_t, PTR);
 char *REmatch(char *, size_t, PTR, size_t *, int);
 void REmprint(PTR, FILE *);
-const char *REs_type(PTR);
-const char *REerror(void);
+#endif
 
 #endif /*  MAWK_REPL_H */
