@@ -1,6 +1,6 @@
 /********************************************
 repl.h
-copyright 2009-2010,2014, Thomas E. Dickey
+copyright 2009-2020,2023, Thomas E. Dickey
 copyright 1991,1993, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,14 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: repl.h,v 1.8 2014/09/14 20:55:12 tom Exp $
- * @Log: repl.h,v @
- * Revision 1.1.1.1  1993/07/03  18:58:19  mike
- * move source to cvs
- *
- * Revision 5.1  1991/12/05  07:59:32  brennan
- * 1.1 pre-release
- *
+ * $MawkId: repl.h,v 1.11 2023/07/20 00:32:26 tom Exp $
  */
 
 /* repl.h */
@@ -29,9 +22,9 @@ the GNU General Public License, version 2, 1991.
 #include "types.h"
 
 typedef struct re_data {
+    PTR compiled;		/* must be first... */
     int anchored;		/* use to limit recursion in gsub */
     int is_empty;		/* check if pattern is empty */
-    PTR compiled;
 } RE_DATA;
 
 /*
@@ -43,10 +36,10 @@ typedef struct re_data {
 #define refRE_DATA(re)  ((PTR) &(re))
 
 PTR re_compile(STRING *);
-char *re_uncompile(PTR);
+STRING *re_uncompile(PTR);
 
 CELL *repl_compile(STRING *);
-char *repl_uncompile(CELL *);
+const STRING *repl_uncompile(CELL *);
 void re_destroy(PTR);
 void repl_destroy(CELL *);
 CELL *replv_cpy(CELL *, CELL *);
